%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Mise en oeuvre d'un estimateur aux moindres carres distribue dans le cas scalaire
% (c'est-a-dire, la variable sous-jacente a estimer, theta, est scalaire). Les "matrices"  
% d'observation sont simplement H_i = 1, i = 1,...,n. Dans cet example, on considere 
% un reseau de n = 6 capteurs specifie par sa matrice d'incidence
%
% TD3, Ex. IV.1 - Surveillance Distribuee de Systemes Multi-agents 

clear 
close all
clc

%%% Parametres de simulation et du probleme %%%

Tf = 60; %% Nombre d'iterations

Delta = 0.2; %% Increment: 0 < Delta < 1 
 
%%% La topologie du reseau de capteurs est specifiee par la matrice d'incidence D
%%% (dans cet exemple le graphe a 6 sommets et 7 aretes: l'orientation des aretes est arbitraire)  

D = [-1 -1  0  0  0  0  0;
      0  1 -1  0  0  0  0;
      1  0  1 -1  0  0  0;
      0  0  0  1  1  1  0;
      0  0  0  0 -1  0 -1;
      0  0  0  0  0 -1  1]; 

%%% On choisit les elements sur la diagonale de la matrice de ponderation en utilisant   
%%% la regle de Metropolis-Hastings (voir le Corollaire dans le Ch. 3, partie 4, du cours).
%%% Ainsi, la condition sur la valeur propre la plus forte de la matrice laplacienne
%%% est automatiquement satisfaite pour tout 0 < Delta < 1
  
W = diag([1/3, 1/2, 1/3, 1/3, 1/3, 1/3, 1/2]); %% Matrice de ponderation 
 
Lw = D*W*D'; %% Matrice laplacienne ponderee  

Mw = eye(6) - Delta*Lw; %% Matrice des "iterations" (ou de Perron)

%%% Verification de la condition sur la valeur propre la plus forte de Lw %%

if max(eig(Lw)) < 2/Delta
   disp('La condition lambda_n(Lw(G)) < 2/Delta est satisfaite !')
else
   disp('La condition lambda_n(Lw(G)) < 2/Delta n''est pas satisfaite !') 
end
   
%%%

theta = 5; %%% Valeur de la variable scalaire a estimer

v = randn(6,1); %%% Bruit guassien additif sur les observations des 6 capteurs

theta_hat_cnt = theta + mean(v); %%% Estime aux moindres carres centralise de theta

%%% L'estime initial de theta des 6 capteurs, theta_hat_i(0), 
%%% est egal aux observations respectives z_i

theta_hat0 = theta*ones(6,1) + v; 
 
%%% Iterations de la methodes des moindres carres distribuee (c'est le protocole de consensus a temps discret)

theta_hat_capteurs = theta_hat0;

for k=1:Tf
    theta_hat_p = Mw^k*theta_hat0;
    theta_hat_capteurs = [theta_hat_capteurs, theta_hat_p]; % Stockage des thetas estimees par les capteurs
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Affichage des resultats %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1)
hold on
plot(0:Tf,theta_hat_capteurs(1,:),'r') %%% Estime aux moindres carres (MC) de theta du capteur 1
plot(0:Tf,theta_hat_capteurs(2,:),'g') %%% Estime aux moindres carres de theta du capteur 2
plot(0:Tf,theta_hat_capteurs(3,:),'b') %%% Estime aux moindres carres de theta du capteur 3
plot(0:Tf,theta_hat_capteurs(4,:),'m') %%% Estime aux moindres carres de theta du capteur 4
plot(0:Tf,theta_hat_capteurs(5,:),'Color',[0.6,0.6,0]) %%% Estime aux moindres carres de theta du capteur 5
plot(0:Tf,theta_hat_capteurs(6,:),'c') %%% Estime aux moindres carres de theta du capteur 6
plot(0:Tf,theta_hat_cnt*ones(Tf+1,1),'k--') %%% Estime aux moindres carres centralise de theta 
plot(0:Tf,theta*ones(Tf+1,1),'k-','LineWidth',1.2) %%% "Vraie" valeur de theta
lgd = legend('\theta_{MC,1}','\theta_{MC,2}','\theta_{MC,3}','\theta_{MC,4}',...
       '\theta_{MC,5}','\theta_{MC,6}','\theta_{MC,central}','\theta_{vraie}');
lgd.FontSize = 13;
xlabel('Iteration')
ylabel('Parametre \theta')
axis([0 Tf min(min(theta_hat_capteurs)) max(max(theta_hat_capteurs))])
